"""
Binance Derivatives Trading Portfolio Margin REST API

OpenAPI Specification for the Binance Derivatives Trading Portfolio Margin REST API
The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""

from enum import Enum


class MarginAccountNewOcoSideEnum(Enum):
    BUY = "BUY"
    SELL = "SELL"


class MarginAccountNewOcoStopLimitTimeInForceEnum(Enum):
    GTC = "GTC"
    IOC = "IOC"
    FOK = "FOK"


class MarginAccountNewOcoNewOrderRespTypeEnum(Enum):
    ACK = "ACK"
    RESULT = "RESULT"


class MarginAccountNewOcoSideEffectTypeEnum(Enum):
    NO_SIDE_EFFECT = "NO_SIDE_EFFECT"
    MARGIN_BUY = "MARGIN_BUY"
    AUTO_REPAY = "AUTO_REPAY"


class ModifyCmOrderSideEnum(Enum):
    BUY = "BUY"
    SELL = "SELL"


class ModifyCmOrderPriceMatchEnum(Enum):
    NONE = "NONE"
    OPPONENT = "OPPONENT"
    OPPONENT_5 = "OPPONENT_5"
    OPPONENT_10 = "OPPONENT_10"
    OPPONENT_20 = "OPPONENT_20"
    QUEUE = "QUEUE"
    QUEUE_5 = "QUEUE_5"
    QUEUE_10 = "QUEUE_10"
    QUEUE_20 = "QUEUE_20"


class ModifyUmOrderSideEnum(Enum):
    BUY = "BUY"
    SELL = "SELL"


class ModifyUmOrderPriceMatchEnum(Enum):
    NONE = "NONE"
    OPPONENT = "OPPONENT"
    OPPONENT_5 = "OPPONENT_5"
    OPPONENT_10 = "OPPONENT_10"
    OPPONENT_20 = "OPPONENT_20"
    QUEUE = "QUEUE"
    QUEUE_5 = "QUEUE_5"
    QUEUE_10 = "QUEUE_10"
    QUEUE_20 = "QUEUE_20"


class NewCmConditionalOrderSideEnum(Enum):
    BUY = "BUY"
    SELL = "SELL"


class NewCmConditionalOrderStrategyTypeEnum(Enum):
    STOP = "STOP"
    STOP_MARKET = "STOP_MARKET"
    TAKE_PROFIT = "TAKE_PROFIT"
    TAKE_PROFIT_MARKET = "TAKE_PROFIT_MARKET"
    TRAILING_STOP_MARKET = "TRAILING_STOP_MARKET"


class NewCmConditionalOrderPositionSideEnum(Enum):
    BOTH = "BOTH"
    LONG = "LONG"
    SHORT = "SHORT"


class NewCmConditionalOrderTimeInForceEnum(Enum):
    GTC = "GTC"
    IOC = "IOC"
    FOK = "FOK"
    GTX = "GTX"


class NewCmConditionalOrderWorkingTypeEnum(Enum):
    MARK_PRICE = "MARK_PRICE"


class NewCmOrderSideEnum(Enum):
    BUY = "BUY"
    SELL = "SELL"


class NewCmOrderTypeEnum(Enum):
    LIMIT = "LIMIT"
    MARKET = "MARKET"


class NewCmOrderPositionSideEnum(Enum):
    BOTH = "BOTH"
    LONG = "LONG"
    SHORT = "SHORT"


class NewCmOrderTimeInForceEnum(Enum):
    GTC = "GTC"
    IOC = "IOC"
    FOK = "FOK"
    GTX = "GTX"


class NewCmOrderPriceMatchEnum(Enum):
    NONE = "NONE"
    OPPONENT = "OPPONENT"
    OPPONENT_5 = "OPPONENT_5"
    OPPONENT_10 = "OPPONENT_10"
    OPPONENT_20 = "OPPONENT_20"
    QUEUE = "QUEUE"
    QUEUE_5 = "QUEUE_5"
    QUEUE_10 = "QUEUE_10"
    QUEUE_20 = "QUEUE_20"


class NewCmOrderNewOrderRespTypeEnum(Enum):
    ACK = "ACK"
    RESULT = "RESULT"


class NewMarginOrderSideEnum(Enum):
    BUY = "BUY"
    SELL = "SELL"


class NewMarginOrderTypeEnum(Enum):
    LIMIT = "LIMIT"
    MARKET = "MARKET"


class NewMarginOrderNewOrderRespTypeEnum(Enum):
    ACK = "ACK"
    RESULT = "RESULT"


class NewMarginOrderSideEffectTypeEnum(Enum):
    NO_SIDE_EFFECT = "NO_SIDE_EFFECT"
    MARGIN_BUY = "MARGIN_BUY"
    AUTO_REPAY = "AUTO_REPAY"


class NewMarginOrderTimeInForceEnum(Enum):
    GTC = "GTC"
    IOC = "IOC"
    FOK = "FOK"
    GTX = "GTX"


class NewMarginOrderSelfTradePreventionModeEnum(Enum):
    NONE = "NONE"
    EXPIRE_TAKER = "EXPIRE_TAKER"
    EXPIRE_BOTH = "EXPIRE_BOTH"
    EXPIRE_MAKER = "EXPIRE_MAKER"


class NewUmConditionalOrderSideEnum(Enum):
    BUY = "BUY"
    SELL = "SELL"


class NewUmConditionalOrderStrategyTypeEnum(Enum):
    STOP = "STOP"
    STOP_MARKET = "STOP_MARKET"
    TAKE_PROFIT = "TAKE_PROFIT"
    TAKE_PROFIT_MARKET = "TAKE_PROFIT_MARKET"
    TRAILING_STOP_MARKET = "TRAILING_STOP_MARKET"


class NewUmConditionalOrderPositionSideEnum(Enum):
    BOTH = "BOTH"
    LONG = "LONG"
    SHORT = "SHORT"


class NewUmConditionalOrderTimeInForceEnum(Enum):
    GTC = "GTC"
    IOC = "IOC"
    FOK = "FOK"
    GTX = "GTX"


class NewUmConditionalOrderWorkingTypeEnum(Enum):
    MARK_PRICE = "MARK_PRICE"


class NewUmConditionalOrderPriceMatchEnum(Enum):
    NONE = "NONE"
    OPPONENT = "OPPONENT"
    OPPONENT_5 = "OPPONENT_5"
    OPPONENT_10 = "OPPONENT_10"
    OPPONENT_20 = "OPPONENT_20"
    QUEUE = "QUEUE"
    QUEUE_5 = "QUEUE_5"
    QUEUE_10 = "QUEUE_10"
    QUEUE_20 = "QUEUE_20"


class NewUmConditionalOrderSelfTradePreventionModeEnum(Enum):
    NONE = "NONE"
    EXPIRE_TAKER = "EXPIRE_TAKER"
    EXPIRE_BOTH = "EXPIRE_BOTH"
    EXPIRE_MAKER = "EXPIRE_MAKER"


class NewUmOrderSideEnum(Enum):
    BUY = "BUY"
    SELL = "SELL"


class NewUmOrderTypeEnum(Enum):
    LIMIT = "LIMIT"
    MARKET = "MARKET"


class NewUmOrderPositionSideEnum(Enum):
    BOTH = "BOTH"
    LONG = "LONG"
    SHORT = "SHORT"


class NewUmOrderTimeInForceEnum(Enum):
    GTC = "GTC"
    IOC = "IOC"
    FOK = "FOK"
    GTX = "GTX"


class NewUmOrderNewOrderRespTypeEnum(Enum):
    ACK = "ACK"
    RESULT = "RESULT"


class NewUmOrderPriceMatchEnum(Enum):
    NONE = "NONE"
    OPPONENT = "OPPONENT"
    OPPONENT_5 = "OPPONENT_5"
    OPPONENT_10 = "OPPONENT_10"
    OPPONENT_20 = "OPPONENT_20"
    QUEUE = "QUEUE"
    QUEUE_5 = "QUEUE_5"
    QUEUE_10 = "QUEUE_10"
    QUEUE_20 = "QUEUE_20"


class NewUmOrderSelfTradePreventionModeEnum(Enum):
    NONE = "NONE"
    EXPIRE_TAKER = "EXPIRE_TAKER"
    EXPIRE_BOTH = "EXPIRE_BOTH"
    EXPIRE_MAKER = "EXPIRE_MAKER"


class QueryUsersCmForceOrdersAutoCloseTypeEnum(Enum):
    LIQUIDATION = "LIQUIDATION"
    ADL = "ADL"


class QueryUsersUmForceOrdersAutoCloseTypeEnum(Enum):
    LIQUIDATION = "LIQUIDATION"
    ADL = "ADL"



"""
Binance Derivatives Trading Portfolio Margin WebSocket Market Streams

OpenAPI Specification for the Binance Derivatives Trading Portfolio Margin WebSocket Market Streams
The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""

from pydantic import BaseModel
from typing import Callable, Optional, TypeVar

from binance_common.configuration import ConfigurationWebSocketStreams
from binance_common.websocket import (
    WebSocketStreamBase,
    WebSocketConnection,
    RequestStreamHandle,
    RequestStream,
)

from .models import UserDataStreamEventsResponse


T = TypeVar("T", bound=BaseModel)


class DerivativesTradingPortfolioMarginWebSocketStreams(WebSocketStreamBase):
    def __init__(
        self,
        configuration: ConfigurationWebSocketStreams,
    ) -> None:

        super().__init__(configuration)
        self.configuration = configuration

    async def ping_server(self, connection: WebSocketConnection):
        """Sends a ping message to the WebSocket server to check the connection status.

        Args:
            connection (WebSocketConnection): The WebSocket connection to send the ping message to.
        """

        if self.session:
            await super().ping_ws_stream(connection=connection)
        else:
            raise ValueError("WebSocket session is not initialized.")

    async def subscribe(self, streams, response_model: Optional[T] = None):
        """Subscribes to the specified WebSocket streams.

        Args:
            streams (list): A list of WebSocket streams to subscribe to.
            response_model (Optional[T]): The Pydantic model to validate the response against.
        """

        await super().subscribe(streams, response_model=response_model)

    async def unsubscribe(self, streams):
        """Unsubscribes from the specified WebSocket streams.

        Args:
            streams (list): A list of WebSocket streams to unsubscribe from.
        """

        await super().unsubscribe(streams)

    def on(self, event: str, callback: Callable[[T], None], stream: str):
        """Registers a callback function for a specific WebSocket event.

        Args:
            event (str): The event type to listen for (e.g., 'message', 'error').
            callback (Callable[[T], None]): The callback function to be called when the event occurs.
            stream (str): The specific WebSocket stream to listen to.
        """

        super().on(event, callback, stream)

    async def list_subscribe(self) -> dict:
        """Lists all currently subscribed WebSocket streams.

        Returns:
            dict: A dictionary containing the current subscriptions.
        """

        return await super().list_subscribe()

    async def close_connection(
        self, connection: WebSocketConnection = None, close_session: bool = True
    ):
        """Closes the WebSocket connection and optionally the session.

        Args:
            connection (WebSocketConnection, optional): The WebSocket connection to close. If None, the current session's connection is used.
            close_session (bool, optional): Whether to close the session after closing the connection. Defaults to True.
        """

        await super().close_connection(connection, close_session)

    async def user_data(self, listenKey: str) -> RequestStreamHandle:
        """Subscribe to User Data Stream (USER_STREAM)

        Args:
            listenKey (str): The listen key for the user data stream.
        Returns:
            RequestStreamHandle: An instance of RequestStream that provides the on() and unsubscribe() methods.
        """

        return await RequestStream(
            self, listenKey, response_model=UserDataStreamEventsResponse
        )



# coding: utf-8

"""
Binance Derivatives Trading Portfolio Margin REST API

OpenAPI Specification for the Binance Derivatives Trading Portfolio Margin REST API
The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt
from typing import Any, ClassVar, Dict, List, Optional
from binance_sdk_derivatives_trading_portfolio_margin.rest_api.models.query_user_negative_balance_auto_exchange_record_response_rows_inner_details_inner import (
    QueryUserNegativeBalanceAutoExchangeRecordResponseRowsInnerDetailsInner,
)
from typing import Set
from typing_extensions import Self


class QueryUserNegativeBalanceAutoExchangeRecordResponseRowsInner(BaseModel):
    """
    QueryUserNegativeBalanceAutoExchangeRecordResponseRowsInner
    """  # noqa: E501

    start_time: Optional[StrictInt] = Field(default=None, alias="startTime")
    end_time: Optional[StrictInt] = Field(default=None, alias="endTime")
    details: Optional[
        List[QueryUserNegativeBalanceAutoExchangeRecordResponseRowsInnerDetailsInner]
    ] = None
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["startTime", "endTime", "details"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def is_array(cls) -> bool:
        return False

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of QueryUserNegativeBalanceAutoExchangeRecordResponseRowsInner from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set(
            [
                "additional_properties",
            ]
        )

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in details (list)
        _items = []
        if self.details:
            for _item_details in self.details:
                if _item_details:
                    _items.append(_item_details.to_dict())
            _dict["details"] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of QueryUserNegativeBalanceAutoExchangeRecordResponseRowsInner from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "startTime": obj.get("startTime"),
                "endTime": obj.get("endTime"),
                "details": (
                    [
                        QueryUserNegativeBalanceAutoExchangeRecordResponseRowsInnerDetailsInner.from_dict(
                            _item
                        )
                        for _item in obj["details"]
                    ]
                    if obj.get("details") is not None
                    else None
                ),
            }
        )
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



"""
Binance Derivatives Trading Portfolio Margin REST API

OpenAPI Specification for the Binance Derivatives Trading Portfolio Margin REST API
The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""

from requests import Session
from binance_common.configuration import ConfigurationRestAPI
from binance_common.models import ApiResponse
from binance_common.signature import Signers
from binance_common.utils import send_request


class MarketDataApi:
    """API Client for MarketDataApi endpoints."""

    def __init__(
        self,
        configuration: ConfigurationRestAPI = None,
        session: Session = None,
        signer: Signers = None,
    ) -> None:
        self._configuration = configuration
        self._session = session
        self._signer = signer

    def test_connectivity(
        self,
    ) -> ApiResponse[None]:
        """
                Test Connectivity
                GET /papi/v1/ping
                https://developers.binance.com/docs/derivatives/portfolio-margin/market-data/Test-Connectivity

                Test connectivity to the Rest API.

        Weight: 1

                Args:

                Returns:
                    ApiResponse[None]

                Raises:
                    RequiredError: If a required parameter is missing.

        """

        body = None
        payload = None

        return send_request(
            self._session,
            self._configuration,
            method="GET",
            path="/papi/v1/ping",
            payload=payload,
            body=body,
            time_unit=self._configuration.time_unit,
        )



"""
Binance Derivatives Trading Portfolio Margin REST API

OpenAPI Specification for the Binance Derivatives Trading Portfolio Margin REST API
The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""

from .rest_api import DerivativesTradingPortfolioMarginRestAPI

__all__ = ["DerivativesTradingPortfolioMarginRestAPI"]



# coding: utf-8

"""
Binance Derivatives Trading Portfolio Margin WebSocket Market Streams

OpenAPI Specification for the Binance Derivatives Trading Portfolio Margin WebSocket Market Streams
The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""


from __future__ import annotations
import json
import pprint
from pydantic import (
    BaseModel,
    ConfigDict,
    ValidationError,
    field_validator,
)
from typing import Any, Optional
from binance_sdk_derivatives_trading_portfolio_margin.websocket_streams.models.account_config_update import (
    AccountConfigUpdate,
)
from binance_sdk_derivatives_trading_portfolio_margin.websocket_streams.models.account_update import (
    AccountUpdate,
)
from binance_sdk_derivatives_trading_portfolio_margin.websocket_streams.models.balanceupdate import (
    Balanceupdate,
)
from binance_sdk_derivatives_trading_portfolio_margin.websocket_streams.models.conditional_order_trade_update import (
    ConditionalOrderTradeUpdate,
)
from binance_sdk_derivatives_trading_portfolio_margin.websocket_streams.models.executionreport import (
    Executionreport,
)
from binance_sdk_derivatives_trading_portfolio_margin.websocket_streams.models.liabilitychange import (
    Liabilitychange,
)
from binance_sdk_derivatives_trading_portfolio_margin.websocket_streams.models.listenkeyexpired import (
    Listenkeyexpired,
)
from binance_sdk_derivatives_trading_portfolio_margin.websocket_streams.models.openorderloss import (
    Openorderloss,
)
from binance_sdk_derivatives_trading_portfolio_margin.websocket_streams.models.order_trade_update import (
    OrderTradeUpdate,
)
from binance_sdk_derivatives_trading_portfolio_margin.websocket_streams.models.outboundaccountposition import (
    Outboundaccountposition,
)
from binance_sdk_derivatives_trading_portfolio_margin.websocket_streams.models.risklevelchange import (
    Risklevelchange,
)
from typing import Union, Set, Dict
from typing_extensions import Self

USERDATASTREAMEVENTSRESPONSE_ONE_OF_SCHEMAS = [
    "AccountConfigUpdate",
    "AccountUpdate",
    "Balanceupdate",
    "ConditionalOrderTradeUpdate",
    "Executionreport",
    "Liabilitychange",
    "Listenkeyexpired",
    "Openorderloss",
    "OrderTradeUpdate",
    "Outboundaccountposition",
    "Risklevelchange",
]


class UserDataStreamEventsResponse(BaseModel):
    """
    UserDataStreamEventsResponse
    """

    # data type: ConditionalOrderTradeUpdate
    oneof_schema_1_validator: Optional[ConditionalOrderTradeUpdate] = None
    # data type: AccountConfigUpdate
    oneof_schema_2_validator: Optional[AccountConfigUpdate] = None
    # data type: AccountUpdate
    oneof_schema_3_validator: Optional[AccountUpdate] = None
    # data type: OrderTradeUpdate
    oneof_schema_4_validator: Optional[OrderTradeUpdate] = None
    # data type: Liabilitychange
    oneof_schema_5_validator: Optional[Liabilitychange] = None
    # data type: Outboundaccountposition
    oneof_schema_6_validator: Optional[Outboundaccountposition] = None
    # data type: Balanceupdate
    oneof_schema_7_validator: Optional[Balanceupdate] = None
    # data type: Executionreport
    oneof_schema_8_validator: Optional[Executionreport] = None
    # data type: Openorderloss
    oneof_schema_9_validator: Optional[Openorderloss] = None
    # data type: Listenkeyexpired
    oneof_schema_10_validator: Optional[Listenkeyexpired] = None
    # data type: Risklevelchange
    oneof_schema_11_validator: Optional[Risklevelchange] = None
    actual_instance: Optional[
        Union[
            AccountConfigUpdate,
            AccountUpdate,
            Balanceupdate,
            ConditionalOrderTradeUpdate,
            Executionreport,
            Liabilitychange,
            Listenkeyexpired,
            Openorderloss,
            OrderTradeUpdate,
            Outboundaccountposition,
            Risklevelchange,
        ]
    ] = None
    one_of_schemas: Set[str] = {
        "AccountConfigUpdate",
        "AccountUpdate",
        "Balanceupdate",
        "ConditionalOrderTradeUpdate",
        "Executionreport",
        "Liabilitychange",
        "Listenkeyexpired",
        "Openorderloss",
        "OrderTradeUpdate",
        "Outboundaccountposition",
        "Risklevelchange",
    }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )

    discriminator_value_class_map: Dict[str, str] = {}

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError(
                    "If a position argument is used, only 1 is allowed to set `actual_instance`"
                )
            if kwargs:
                raise ValueError(
                    "If a position argument is used, keyword arguments cannot be used."
                )
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator("actual_instance")
    def actual_instance_must_validate_oneof(cls, v):

        error_messages = []
        match = 0
        # validate data type: ConditionalOrderTradeUpdate
        if not isinstance(v, ConditionalOrderTradeUpdate):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `ConditionalOrderTradeUpdate`"
            )
        else:
            match += 1
        # validate data type: AccountConfigUpdate
        if not isinstance(v, AccountConfigUpdate):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `AccountConfigUpdate`"
            )
        else:
            match += 1
        # validate data type: AccountUpdate
        if not isinstance(v, AccountUpdate):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `AccountUpdate`"
            )
        else:
            match += 1
        # validate data type: OrderTradeUpdate
        if not isinstance(v, OrderTradeUpdate):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `OrderTradeUpdate`"
            )
        else:
            match += 1
        # validate data type: Liabilitychange
        if not isinstance(v, Liabilitychange):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `Liabilitychange`"
            )
        else:
            match += 1
        # validate data type: Outboundaccountposition
        if not isinstance(v, Outboundaccountposition):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `Outboundaccountposition`"
            )
        else:
            match += 1
        # validate data type: Balanceupdate
        if not isinstance(v, Balanceupdate):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `Balanceupdate`"
            )
        else:
            match += 1
        # validate data type: Executionreport
        if not isinstance(v, Executionreport):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `Executionreport`"
            )
        else:
            match += 1
        # validate data type: Openorderloss
        if not isinstance(v, Openorderloss):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `Openorderloss`"
            )
        else:
            match += 1
        # validate data type: Listenkeyexpired
        if not isinstance(v, Listenkeyexpired):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `Listenkeyexpired`"
            )
        else:
            match += 1
        # validate data type: Risklevelchange
        if not isinstance(v, Risklevelchange):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `Risklevelchange`"
            )
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError(
                "Multiple matches found when setting `actual_instance` in UserDataStreamEventsResponse with oneOf schemas: AccountConfigUpdate, AccountUpdate, Balanceupdate, ConditionalOrderTradeUpdate, Executionreport, Liabilitychange, Listenkeyexpired, Openorderloss, OrderTradeUpdate, Outboundaccountposition, Risklevelchange. Details: "
                + ", ".join(error_messages)
            )
        elif match == 0:
            # no match
            raise ValueError(
                "No match found when setting `actual_instance` in UserDataStreamEventsResponse with oneOf schemas: AccountConfigUpdate, AccountUpdate, Balanceupdate, ConditionalOrderTradeUpdate, Executionreport, Liabilitychange, Listenkeyexpired, Openorderloss, OrderTradeUpdate, Outboundaccountposition, Risklevelchange. Details: "
                + ", ".join(error_messages)
            )
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into ConditionalOrderTradeUpdate
        try:
            instance.actual_instance = ConditionalOrderTradeUpdate.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into AccountConfigUpdate
        try:
            instance.actual_instance = AccountConfigUpdate.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into AccountUpdate
        try:
            instance.actual_instance = AccountUpdate.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into OrderTradeUpdate
        try:
            instance.actual_instance = OrderTradeUpdate.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into Liabilitychange
        try:
            instance.actual_instance = Liabilitychange.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into Outboundaccountposition
        try:
            instance.actual_instance = Outboundaccountposition.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into Balanceupdate
        try:
            instance.actual_instance = Balanceupdate.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into Executionreport
        try:
            instance.actual_instance = Executionreport.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into Openorderloss
        try:
            instance.actual_instance = Openorderloss.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into Listenkeyexpired
        try:
            instance.actual_instance = Listenkeyexpired.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into Risklevelchange
        try:
            instance.actual_instance = Risklevelchange.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError(
                "Multiple matches found when deserializing the JSON string into UserDataStreamEventsResponse with oneOf schemas: AccountConfigUpdate, AccountUpdate, Balanceupdate, ConditionalOrderTradeUpdate, Executionreport, Liabilitychange, Listenkeyexpired, Openorderloss, OrderTradeUpdate, Outboundaccountposition, Risklevelchange. Details: "
                + ", ".join(error_messages)
            )
        elif match == 0:
            # no match
            raise ValueError(
                "No match found when deserializing the JSON string into UserDataStreamEventsResponse with oneOf schemas: AccountConfigUpdate, AccountUpdate, Balanceupdate, ConditionalOrderTradeUpdate, Executionreport, Liabilitychange, Listenkeyexpired, Openorderloss, OrderTradeUpdate, Outboundaccountposition, Risklevelchange. Details: "
                + ", ".join(error_messages)
            )
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(
            self.actual_instance.to_json
        ):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(
        self,
    ) -> Optional[
        Union[
            Dict[str, Any],
            AccountConfigUpdate,
            AccountUpdate,
            Balanceupdate,
            ConditionalOrderTradeUpdate,
            Executionreport,
            Liabilitychange,
            Listenkeyexpired,
            Openorderloss,
            OrderTradeUpdate,
            Outboundaccountposition,
            Risklevelchange,
        ]
    ]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(
            self.actual_instance.to_dict
        ):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())



"""
Binance Derivatives Trading Portfolio Margin WebSocket Market Streams

OpenAPI Specification for the Binance Derivatives Trading Portfolio Margin WebSocket Market Streams
The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""

from .account_config_update import AccountConfigUpdate as AccountConfigUpdate
from .account_config_update_ac import AccountConfigUpdateAc as AccountConfigUpdateAc
from .account_update import AccountUpdate as AccountUpdate
from .account_update_a import AccountUpdateA as AccountUpdateA
from .account_update_ab_inner import AccountUpdateABInner as AccountUpdateABInner
from .account_update_ap_inner import AccountUpdateAPInner as AccountUpdateAPInner
from .balanceupdate import Balanceupdate as Balanceupdate
from .conditional_order_trade_update import (
    ConditionalOrderTradeUpdate as ConditionalOrderTradeUpdate,
)
from .conditional_order_trade_update_so import (
    ConditionalOrderTradeUpdateSo as ConditionalOrderTradeUpdateSo,
)
from .executionreport import Executionreport as Executionreport
from .liabilitychange import Liabilitychange as Liabilitychange
from .listenkeyexpired import Listenkeyexpired as Listenkeyexpired
from .openorderloss import Openorderloss as Openorderloss
from .openorderloss_o_inner import OpenorderlossOInner as OpenorderlossOInner
from .order_trade_update import OrderTradeUpdate as OrderTradeUpdate
from .order_trade_update_o import OrderTradeUpdateO as OrderTradeUpdateO
from .outboundaccountposition import Outboundaccountposition as Outboundaccountposition
from .outboundaccountposition_b_inner import (
    OutboundaccountpositionBInner as OutboundaccountpositionBInner,
)
from .risklevelchange import Risklevelchange as Risklevelchange
from .user_data_stream_events_response import (
    UserDataStreamEventsResponse as UserDataStreamEventsResponse,
)



from binance_sdk_derivatives_trading_portfolio_margin.derivatives_trading_portfolio_margin import (
    DerivativesTradingPortfolioMargin,
)
from binance_common.errors import (
    ClientError,
    RequiredError,
    UnauthorizedError,
    ForbiddenError,
    TooManyRequestsError,
    RateLimitBanError,
    ServerError,
    NetworkError,
    NotFoundError,
    BadRequestError,
)
from binance_common.constants import (
    DERIVATIVES_TRADING_PORTFOLIO_MARGIN_REST_API_PROD_URL,
    DERIVATIVES_TRADING_PORTFOLIO_MARGIN_REST_API_TESTNET_URL,
    DERIVATIVES_TRADING_PORTFOLIO_MARGIN_WS_STREAMS_PROD_URL,
    DERIVATIVES_TRADING_PORTFOLIO_MARGIN_WS_STREAMS_TESTNET_URL,
)

__all__ = [
    "DerivativesTradingPortfolioMargin",
    "DERIVATIVES_TRADING_PORTFOLIO_MARGIN_REST_API_PROD_URL",
    "DERIVATIVES_TRADING_PORTFOLIO_MARGIN_REST_API_TESTNET_URL",
    "DERIVATIVES_TRADING_PORTFOLIO_MARGIN_WS_STREAMS_PROD_URL",
    "DERIVATIVES_TRADING_PORTFOLIO_MARGIN_WS_STREAMS_TESTNET_URL",
    "ClientError",
    "RequiredError",
    "UnauthorizedError",
    "ForbiddenError",
    "TooManyRequestsError",
    "RateLimitBanError",
    "ServerError",
    "NetworkError",
    "NotFoundError",
    "BadRequestError",
]



# coding: utf-8

"""
Binance Derivatives Trading Portfolio Margin REST API

OpenAPI Specification for the Binance Derivatives Trading Portfolio Margin REST API
The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Set
from typing_extensions import Self


class NewUmOrderResponse(BaseModel):
    """
    NewUmOrderResponse
    """  # noqa: E501

    client_order_id: Optional[StrictStr] = Field(default=None, alias="clientOrderId")
    cum_qty: Optional[StrictStr] = Field(default=None, alias="cumQty")
    cum_quote: Optional[StrictStr] = Field(default=None, alias="cumQuote")
    executed_qty: Optional[StrictStr] = Field(default=None, alias="executedQty")
    order_id: Optional[StrictInt] = Field(default=None, alias="orderId")
    avg_price: Optional[StrictStr] = Field(default=None, alias="avgPrice")
    orig_qty: Optional[StrictStr] = Field(default=None, alias="origQty")
    price: Optional[StrictStr] = None
    reduce_only: Optional[StrictBool] = Field(default=None, alias="reduceOnly")
    side: Optional[StrictStr] = None
    position_side: Optional[StrictStr] = Field(default=None, alias="positionSide")
    status: Optional[StrictStr] = None
    symbol: Optional[StrictStr] = None
    time_in_force: Optional[StrictStr] = Field(default=None, alias="timeInForce")
    type: Optional[StrictStr] = None
    self_trade_prevention_mode: Optional[StrictStr] = Field(
        default=None, alias="selfTradePreventionMode"
    )
    good_till_date: Optional[StrictInt] = Field(default=None, alias="goodTillDate")
    update_time: Optional[StrictInt] = Field(default=None, alias="updateTime")
    price_match: Optional[StrictStr] = Field(default=None, alias="priceMatch")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = [
        "clientOrderId",
        "cumQty",
        "cumQuote",
        "executedQty",
        "orderId",
        "avgPrice",
        "origQty",
        "price",
        "reduceOnly",
        "side",
        "positionSide",
        "status",
        "symbol",
        "timeInForce",
        "type",
        "selfTradePreventionMode",
        "goodTillDate",
        "updateTime",
        "priceMatch",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def is_array(cls) -> bool:
        return False

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of NewUmOrderResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set(
            [
                "additional_properties",
            ]
        )

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of NewUmOrderResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "clientOrderId": obj.get("clientOrderId"),
                "cumQty": obj.get("cumQty"),
                "cumQuote": obj.get("cumQuote"),
                "executedQty": obj.get("executedQty"),
                "orderId": obj.get("orderId"),
                "avgPrice": obj.get("avgPrice"),
                "origQty": obj.get("origQty"),
                "price": obj.get("price"),
                "reduceOnly": obj.get("reduceOnly"),
                "side": obj.get("side"),
                "positionSide": obj.get("positionSide"),
                "status": obj.get("status"),
                "symbol": obj.get("symbol"),
                "timeInForce": obj.get("timeInForce"),
                "type": obj.get("type"),
                "selfTradePreventionMode": obj.get("selfTradePreventionMode"),
                "goodTillDate": obj.get("goodTillDate"),
                "updateTime": obj.get("updateTime"),
                "priceMatch": obj.get("priceMatch"),
            }
        )
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj
